/*
    Robot servo controller

    Sends contrller data to the server

    (C) 2004 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: huntjas2@msu.edu

    Compile with -lwsock32

    FILE: network.cpp
*/


#include <windows.h>
#include <stdio.h>
#include "network.h"
#include "display.h"

char host_name[256];


SOCKET server_socket;

int rec_data = 0;           // true if date received
int connected = 0;          // true if date connected
int attempting_connect = 0;
int no_connect = 0;         // true if we do not want to connect


/************************
*** Network Functions ***
************************/

/* configure and init the network */
void startup_network(HWND hwnd)
{
    add_line("Starting network.");

    // fire up winsock
    WSADATA wsda;
    WSAStartup(MAKEWORD(1, 1),&wsda);

    add_line("Startup complete.");
}
/* end of startup network */



/* set the server ost name */
void set_host_name(char *name)
{
    strcpy(host_name, name);
}
/* end of set_host_name */


/* attept to connect to te server */
void connect_to_server( void )
{
    char out[256];

    sprintf(out, "Connecting to %s", host_name);

    add_line(out);

    // create the socket
    server_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

    if (server_socket == INVALID_SOCKET)
    {
        add_line("Socket error.");
        return;
    }

    LPHOSTENT hostent;

    hostent = gethostbyname(host_name);

    if (!hostent)
    {
        add_line("Host name error.");
        return;
    }

    /* setup address information */
    SOCKADDR_IN server;

    server.sin_addr=*((LPIN_ADDR)*hostent->h_addr_list);
    server.sin_port=htons(PORT);
    server.sin_family=AF_INET;

    WSAAsyncSelect(server_socket, main_win, WM_NETEVENT, FD_WRITE | FD_CONNECT |
                                                FD_READ | FD_CLOSE);

    connected = 0;
    attempting_connect = 1;

    connect(server_socket,(LPSOCKADDR)&server, sizeof(struct sockaddr));
}
/* end of connect_to_server */



/* shutdown win sock */
void shutdown_network(void)
{
    closesocket(server_socket);     // close server socket

    WSACleanup();
}
/* end of shutdown_network */



/* set controller to a contol set */
void show_command(char *comm)
{
    char out[1024];

    sprintf(out, "Return: Left: %c Right %c Pan: %d Tilt: %d",
                comm[1], comm[2],
                (unsigned char) comm[3],
                (unsigned char) comm[4]);

    add_line(out);
}
/* end of set controller */


void read_server_data(void)
{
    char buffer[rec_buffer_size]; // receive buffer
    char packet[5];

    int n_bytes = recv(server_socket, buffer, rec_buffer_size, 0);

    // read the entire buffer
    for (int n = 0; n < n_bytes / 5; n++)
    {
        memcpy(packet, buffer + n * 5, 5);
        show_command(packet);
    }
}
/* end of read_client_data */




/* send a control packet to the server */
void send_packet(char *packet, unsigned int size)
{
    if (send(server_socket, packet, size, 0) == SOCKET_ERROR)
    {
        add_line("Unable to send data");
        connected = 0;
    }
}
/* end of set_packet */


/* send the controller values to the server */
void send_controller_values()
{
    if (!connected) return;

    char out[100];
    sprintf(out, "Left: %c Right %c Pan: %d Tilt: %d", l_motor, r_motor, pan, tilt);
    add_line(out);

    char data_packet[5]; // the data packet

    data_packet[0] = 0x01; // command byte data (01 == update movement data)
    data_packet[1] = l_motor;
    data_packet[2] = r_motor;
    data_packet[3] = pan;
    data_packet[4] = tilt;

    send_packet(data_packet, 5);
}
/* end of send_controller_values */


/* send keepalive data */
void send_keep_alive( void )
{
    char data_packet[5]; // the data packet

    memset(data_packet, 0, 5); // make the packet all zeros

    send_packet(data_packet, 5);
}
/* end of send_control_state */



/* connect to server */
void connect(void)
{
    closesocket(server_socket);
    connected = 0;
    no_connect = 0;
}
/* end of conenct to server */


/* disconnect from server */
void disconnect(void)
{
    closesocket(server_socket);
    connected = 0;
    no_connect = 1;
    add_line("Disconnect.");
}
/* end of disconnect */



/* get a hostname from the user */
void get_servername( void )
{
/*    char tmp_name[128];

    HWND hdialog;

    if (GetDlgItemText(NULL, ID_ITEMNAME, tmp_name, 128))
        set_host_name(tmp_name);*/
}
/* end of get hostname */


/* deal with network messages */
void network_message(WPARAM wParam, LPARAM lParam)
{
    switch (WSAGETSELECTEVENT(lParam))
    {
        case FD_ACCEPT:
            break;

        case FD_CONNECT:
            if(WSAGETSELECTERROR(lParam))
            {
                add_line("Time out.");
                attempting_connect = 0;
                closesocket(server_socket);
            }
            else
            {
                add_line("Connected.");
                connected = 1;
                attempting_connect = 0;
            }
            break;

        case FD_CLOSE:
            add_line("Server disconnect.");
            connected = 0;
            closesocket(server_socket);
            break;

        // data back from server
        case FD_READ:
            read_server_data();
            break;

        case FD_WRITE:
            break;
    }
}
/* end of network_message */


/*******************************
*** End of Network Functions ***
*******************************/


